
#include "wobbler.h"
#include "clipper.h"

#include <GL/gl.h>
#include <GL/glu.h>

#include <glib.h>
#include "node.h"
#include "cube.h"
#include <math.h>

#define N_TILES 4

static void
wobbler_node_render (Node *node)
{
    WobblerNode *wobbler_node = (WobblerNode *)node;
    GList *list;
    int i, j, k;
    double x, y;
    
    /* coordinate system */
    
    i = 0;
    j = 0;
    k = 0;
    
    for (list = wobbler_node->clips; list != NULL; list = list->next)
    {
	Node *child = list->data;
	
	x = (1.0 / N_TILES) * i + (1.0 / N_TILES) / 2;
	y = (1.0 / N_TILES) * j + (1.0 / N_TILES) / 2;
	
	glEnable (GL_TEXTURE_2D);
	
	glPushMatrix();
	
	if (i == 1 && j == 1)
	{
	    glTranslatef (x, y, 0);

	    glScalef (0.25, 0.25, 0.25);

	    glColor4f (1 * 0.5 * sin (node_get_time (node)) + 1, 1 * 0.3, 1 * 0.3, 0.9);
	
#if 0
	    glDisable (GL_DEPTH_TEST);
	    glEnable (GL_BLEND);
	    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	
	    
	    glBegin (GL_QUADS);
	    
	    glVertex2f (0.0, 1.0);
	    glVertex2f (1.0, 1.0);
	    glVertex2f (1.0, 0.0);
	    glVertex2f (0.0, 0.0);
	    
	    glEnd ();
#endif

	    glTranslatef (-0.5, -0.5, 0);

	    glTranslatef (0.0, 0.0, 0.2 * sin (5 * node_get_time (node)));
	    /* coordinate system now has origo in the corner */
	    
	    glTranslatef (0.5, 0.5, 0.5);
	    
	    glRotatef (180 * node_get_time (node), 0.0, 0.0, 1.0);

	    glRotatef (60 * node_get_time (node), 0.0, 1.0, 0.0);
	    glRotatef (90 * node_get_time (node), 1.0, 0.0, 0.0);

	    glTranslatef (-0.5, -0.5, -0.5);
	    
#if 0
#endif
#if 0
	    glTranslatef (-0.5, -0.5, -0.5);
#endif
	    
#if 0
	    glTranslatef (-0.0, -1.0, 0);
#endif
	    
#if 0
	    glTranslatef (-1.5, -1.5, 0);
#endif
	    

#if 0
#endif
	    
	    child->render (child);
#if 0
	    glRotatef (60 * node_get_time (node), 0.0, 1.0, 0.0);
#endif
	
	    glTranslatef (-x, -y, 0.0);
	}
	else
	    child->render (child);
	
	glPopMatrix();
	
	glColor4f (k * 0.3, k * 0.3, k * 0.3, 0.3);
	
	glDisable (GL_TEXTURE_2D);
	
	glDisable (GL_DEPTH_TEST);
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	
	
#if 0
	glBegin (GL_QUADS);
	
	glVertex2f (x - 0.1, y - 0.1);
	glVertex2f (x - 0.1, y + 0.1);
	glVertex2f (x + 0.1, y + 0.1);
	glVertex2f (x + 0.1, y - 0.1);
	
	glEnd ();
#endif
	
	glEnable (GL_DEPTH_TEST);
	
	j++;
	if (j == N_TILES)
	{
	    j = 0;
	    i++;
	}
	k++;
    }
    
#if 0
    glDisable (GL_DEPTH_TEST);
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	
    
    glBegin (GL_QUADS);
    
    glVertex2f (x - 0.1, y - 0.1);
    glVertex2f (x - 0.1, y + 0.1);
    glVertex2f (x + 0.1, y + 0.1);
    glVertex2f (x + 0.1, y - 0.1);
    
    glEnd ();
#endif
    
#if 0
    g_print ("asdf\n");
#endif
}

WobblerNode *
wobbler_node_new (Node *child)
{
    WobblerNode *wobbler = g_new0 (WobblerNode, 1);
    int i;
    int j;
    
    wobbler->child = child;
    
    for (i = 0; i < N_TILES; ++i)
	for (j = 0; j < N_TILES; ++j)
	{
	    Node *node;

	    if (i == 1 && j == 1)
	    {
		ClipNode *clip_node = clip_node_new (
		    (1.0 / N_TILES) * i, (1.0 / N_TILES) * j,
		    (1.0 / N_TILES), (1.0 / N_TILES), child);
		CubeNode *cube;

		cube = cube_node_new ();

		clip_node = (ClipNode *)child;
		
		cube_set_face (cube, 0, child);
		cube_set_face (cube, 1, child);
		cube_set_face (cube, 2, child);
		cube_set_face (cube, 3, child);
		cube_set_face (cube, 4, child);
		cube_set_face (cube, 5, child);

		node = (Node *)cube;
	    }
	    else
	    {
		node = (Node *)clip_node_new (
		    (1.0 / N_TILES) * i, (1.0 / N_TILES) * j,
		    (1.0 / N_TILES), (1.0 / N_TILES), child);
	    }
	    
	    wobbler->clips = g_list_append (wobbler->clips, node);
	}
    
    wobbler->node.render = wobbler_node_render;

    return wobbler;
}
