#ifndef WS_H
#define WS_H

#include <glib.h>

#include <X11/X.h>
#include <X11/Xlib.h>

typedef struct Ws Ws;
typedef struct WsClass WsClass;
typedef struct WsScreen WsScreen;
typedef struct WsScreenClass WsScreenClass;
typedef struct WsDrawable WsDrawable;
typedef struct WsDrawableClass WsDrawableClass;
typedef struct WsWindow WsWindow;
typedef struct WsWindowClass WsWindowClass;
typedef struct WsRegion WsRegion;
typedef struct WsPixmap WsPixmap;
typedef struct WsPixmapClass WsPixmapClass;
typedef struct WsShmPixmap WsShmPixmap;
typedef struct WsShmPixmapClass WsShmPixmapClass;

typedef struct
{
    int x, y;
    guint width, height;
} WsRectangle;

typedef struct
{
    gint alpha;
    gint red;
    gint green;
    gint blue;
} WsColor;

typedef enum
{
    WS_FORMAT_UNKNOWN,
    WS_FORMAT_RGB_16,
    WS_FORMAT_RGB_24,
    WS_FORMAT_RGB_32,
    WS_FORMAT_ARGB_32
} WsFormat;

/* WS */
Ws *      ws_new                (const char *dpy);
void	  ws_process_xevent	(Ws	    *ws,
				 XEvent      *xev);
void      ws_process_error      (Ws	    *ws,
				 XErrorEvent *error);
void      ws_grab               (Ws         *ws);
void      ws_ungrab             (Ws         *ws);
gboolean  ws_init_composite     (Ws         *ws);
gboolean  ws_init_damage        (Ws         *ws);
gboolean  ws_init_fixes         (Ws         *ws);
gboolean  ws_init_test		(Ws	    *ws);
GList *   ws_list_screens       (Ws         *ws);
void      ws_sync               (Ws         *ws);
void      ws_sync_with_xdisplay (Ws         *ws,
				 gpointer    dpy);
void      ws_flush              (Ws         *ws);
void      ws_set_synchronize    (Ws         *ws,
				 gboolean    sync);
void      ws_set_ignore_grabs   (Ws	    *ws,
				 gboolean    ignore);
void	  ws_begin_error_trap  (Ws	    *ws);
void	  ws_end_error_trap    (Ws	    *ws);

/* WsScreen */
WsScreen *ws_screen_get_default     (Ws         *ws);
WsScreen *ws_screen_get_from_number (Ws *ws,
				     int number);
WsWindow *ws_screen_get_root_window (WsScreen *screen);
int       ws_screen_get_number      (WsScreen *screen);
void      ws_screen_graphics_sync (WsScreen *screen);

/* WsDrawable */
typedef void (* WsWindowDamageCallback) (WsDrawable *window,
					 gpointer    data);
Ws *      ws_drawable_get_ws              (WsDrawable             *window);
WsScreen *ws_drawable_query_screen        (WsDrawable             *window);
void      ws_drawable_set_damage_callback (WsDrawable             *window,
					   WsWindowDamageCallback  cb,
					   gpointer                data);
WsRegion *ws_drawable_undamage            (WsDrawable             *window);
void      ws_drawable_copy_area           (WsDrawable             *src,
					   int                     src_x,
					   int                     src_y,
					   int                     width,
					   int                     height,
					   WsDrawable             *dst,
					   int                     dst_x,
					   int                     dst_y,
					   WsRegion		  *clip);
gboolean  ws_drawable_query_geometry      (WsDrawable		  *drawable,
					   WsRectangle		  *rect);
int	  ws_drawable_query_depth (WsDrawable *drawable);

/* WsPixmap */
WsPixmap *ws_pixmap_new (WsDrawable *drawable,
			 int	     width,
			 int	     height,
			 int	     depth);

/* WsShmPixmap */
WsShmPixmap * ws_shm_pixmap_new      (WsDrawable  *window,
				      int          width,
				      int          height,
				      WsFormat     format);
const guchar *ws_shm_pixmap_get_data (WsShmPixmap *pixmap);
void          ws_shm_pixmap_unref    (WsShmPixmap *pixmap);
WsFormat      ws_shm_pixmap_get_format (WsShmPixmap *pixmap);
guint	      ws_shm_pixmap_get_bytes_per_row (WsShmPixmap *pixmap);

/* WsWindow */
typedef void (* WsConfigureCallback) (WsWindow *window,
				      int	x,
				      int	y,
				      int	width,
				      int	height,
				      int	border_width,
				      WsWindow *above,
				      gboolean  override_redirect,
				      gpointer  data);

void      ws_window_raise                 (WsWindow    *window);
void      ws_window_map                   (WsWindow    *window);
WsWindow *ws_window_new                   (WsWindow    *parent);
WsWindow *ws_window_new_gl                (WsWindow    *parent);
GList *   ws_window_query_subwindows      (WsWindow    *window);
void      ws_window_unredirect            (WsWindow    *window);
void      ws_window_redirect              (WsWindow    *window);
void      ws_window_set_override_redirect (WsWindow    *window,
					   gboolean     override_redirect);
WsPixmap *ws_window_name_pixmap		  (WsWindow    *window);
void      ws_window_redirect_subwindows   (WsWindow    *window);
void      ws_window_setup_gl              (WsWindow    *window);
void      ws_window_gl_begin              (WsWindow    *window);
void      ws_window_gl_end                (WsWindow    *window);
void      ws_window_gl_swap_buffers       (WsWindow    *window);
gboolean  ws_window_query_input_only      (WsWindow    *window);
gboolean  ws_window_query_viewable	  (WsWindow    *window);
WsFormat  ws_window_query_format	  (WsWindow    *window);
char *	  ws_window_query_title           (WsWindow    *window);
void	  ws_window_set_input_shape	  (WsWindow    *window,
					   WsRegion    *shape);
WsWindow *ws_window_lookup		  (Ws	       *ws,
					   Window	xid);
void	  ws_window_set_configure_callback (WsWindow   *window,
					    WsConfigureCallback cb,
					    gpointer	data);

/* WsRegion */
WsRegion *ws_region_new   (Ws       *ws);
void      ws_region_unref (WsRegion *region);

#endif
