#include "wsint.h"

void
ws_drawable_fill_area (WsDrawable *drawable,
		       int x, int y, int width, int height,
		       const WsColor *color)
{
#if 0
    XrenderColor rcolor;
#endif

    /* FIXME implement this - we really need a format for
     * every drawable
     */
}

void
ws_drawable_copy_area (WsDrawable *src,
		       int sx, int sy, int width, int height,
		       WsDrawable *dest,
		       int dx, int dy,
		       WsRegion *clip)
{
    Display *xdisplay = src->ws->xdisplay;
    
    GC gc = XCreateGC (xdisplay, src->xid, 0, NULL);

#if 0
    if (clip)
	XFixesSetGCClipRegion (xdisplay, gc, 0, 0, clip->xregion);
#endif

    XSetForeground (xdisplay, gc, 0x00FFFF00);
    
#if 0
    g_print ("copying from %lx to %lx\n", src->xid, dest->xid);
#endif

    XFillRectangle (xdisplay, dest->xid, gc, 0, 0, width, height);
    
    XCopyArea (xdisplay,
	       src->xid, dest->xid, gc,
	       sx, sy, width, height, dx, dy);

    XFreeGC (src->ws->xdisplay, gc);
}

static gboolean
drawable_query_geometry_internal (WsDrawable *drawable,
				  Window     *root,
				  int        *x,
				  int        *y,
				  guint      *width,
				  guint      *height,
				  guint      *depth)
{
    int x_internal, y_internal;
    Window root_internal;
    guint width_internal, height_internal, depth_internal;
    guint bw_dummy;

    g_return_val_if_fail (drawable != NULL, FALSE);

    if (!XGetGeometry (
	    drawable->ws->xdisplay, drawable->xid, &root_internal,
	    &x_internal, &y_internal,
	    &width_internal, &height_internal, &bw_dummy, &depth_internal))
    {
	return FALSE;
    }
	
    if (root)
	*root = root_internal;

    if (x)
	*x = x_internal;

    if (y)
	*y = y_internal;

    if (width)
	*width = width_internal;

    if (height)
	*height = height_internal;

    if (depth)
	*depth = depth_internal;

    return TRUE;
}

/* Based on code found in Xmu */
static Screen *
drawable_get_xscreen (WsDrawable *drawable)
{
    Display *xdisplay;
    Window root;
    int i;

    xdisplay = drawable->ws->xdisplay;
    
    drawable_query_geometry_internal (drawable, &root, NULL, NULL, NULL, NULL, NULL);

    for (i = 0; i < ScreenCount (xdisplay); i++)
    {
        if (root == RootWindow (xdisplay, i))
	    return ScreenOfDisplay (xdisplay, i);
    }
    
    return NULL;
}

Ws *
ws_drawable_get_ws (WsDrawable *drawable)
{
    g_return_val_if_fail (drawable != NULL, NULL);
    
    return drawable->ws;
}

WsScreen *
ws_drawable_query_screen (WsDrawable *drawable)
{
    int i;
    Screen *xscreen = drawable_get_xscreen (drawable);

    for (i = 0; i < drawable->ws->n_screens; ++i)
    {
	if (drawable->ws->screens[i]->xscreen == xscreen)
	    return drawable->ws->screens[i];
    }
    
    return NULL;
}

void
ws_drawable_set_damage_callback (WsDrawable *drawable,
				 WsWindowDamageCallback cb,
				 gpointer	 data)
{
    g_return_if_fail (drawable->ws->damage.available);
    
    drawable->damage_cb = cb;
    drawable->damage_data = data;

    if (drawable->damage_cb && !drawable->xdamage)
    {
	drawable->xdamage = XDamageCreate (drawable->ws->xdisplay,
						  drawable->xid,
					 XDamageReportNonEmpty);

	ws_drawable_undamage (drawable);

	ws_sync (drawable->ws);
    }
    
    if (!drawable->damage_cb && drawable->xdamage)
    {
	XDamageDestroy (drawable->ws->xdisplay, drawable->xdamage);
	drawable->xdamage = None;
    }
}

WsRegion *
ws_drawable_undamage (WsDrawable *drawable)
{
    WsRegion *region = ws_region_new (drawable->ws);
    
    XDamageSubtract (drawable->ws->xdisplay, drawable->xdamage,
		     None, region->xregion);
    
    return region;
}

void
_ws_drawable_process_event (WsDrawable *drawable,
			    XEvent     *xevent)
{
    if (drawable->ws->damage.available &&
	xevent->type == drawable->ws->damage.event_base + XDamageNotify)
    {
	if (drawable->damage_cb)
	    drawable->damage_cb (drawable, drawable->damage_data);
    }
}

gboolean
ws_drawable_query_geometry (WsDrawable *drawable,
			    WsRectangle *rectangle)
{
    Window root;		/* dummy */
    guint bw, depth;		/* dummies */
    WsRectangle rect;
    
    if (!XGetGeometry (
	    drawable->ws->xdisplay, drawable->xid,
	    &root,
	    &rect.x, &rect.y,
	    &rect.width, &rect.height,
	    &bw, &depth))
    {
	return FALSE;
    }
    else
    {
	if (rectangle)
	    *rectangle = rect;

	return TRUE;
    }
}

int
ws_drawable_query_depth (WsDrawable *drawable)
{
    guint depth;
    
    if (drawable_query_geometry_internal (
	    drawable, NULL, NULL, NULL, NULL, NULL, &depth))
    {
	return depth;
    }

    return -1;
}

void
ws_drawable_free (WsDrawable *drawable)
{
    if (drawable->xdamage)
    {
	ws_begin_error_trap (drawable->ws);
	
	XDamageDestroy (drawable->ws->xdisplay, drawable->xdamage);
	drawable->xdamage = None;
	
	ws_end_error_trap (drawable->ws);
    }
}
