#include "ws.h"
#include "wsint.h"

#include <X11/X.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xcomposite.h>
#include <X11/extensions/Xdamage.h>

static WsScreen *
screen_new (Ws     *ws,
	    Screen *xscreen)
{
    WsScreen *screen = g_new0 (WsScreen, 1);
    
    screen->ws = ws;
    screen->xscreen = xscreen;
    
    return screen;
}

WsScreen *
ws_screen_get_default (Ws *ws)
{
    Screen *xscreen;
    int screen_no;
    int i;
    
    g_return_val_if_fail (ws != NULL, NULL);
    
    screen_no = DefaultScreen (ws->xdisplay);
    xscreen = ScreenOfDisplay (ws->xdisplay, screen_no);
    
    for (i = 0; i < ws->n_screens; ++i)
    {
	if (ws->screens[i]->xscreen == xscreen)
	    return ws->screens[i];
    }
    
    return NULL;
}

WsWindow *
ws_screen_get_root_window (WsScreen *screen)
{
    Window root = RootWindowOfScreen (screen->xscreen);
    
    return _ws_window_ensure (screen->ws, root);
}

int
ws_screen_get_number (WsScreen *screen)
{
    int i;
    
    g_return_val_if_fail (screen != NULL, -1);
    
    for (i = 0; i < screen->ws->n_screens; ++i)
	if (screen->ws->screens[i] == screen)
	    return i;
    
    return -1;
}

WsScreen *
ws_screen_get_from_number (Ws *ws,
			   int number)
{
    if (!ws->screens[number])
    {
	ws->screens[number] = screen_new (
	    ws, ScreenOfDisplay (ws->xdisplay, number));
    }

    return ws->screens[number];

}
			    

void
ws_screen_graphics_sync (WsScreen *screen)
{
    WsWindow *root = ws_screen_get_root_window (screen);
    XImage *tmp_image;

    tmp_image = XGetImage (screen->ws->xdisplay, root->drawable.xid,
			   0, 0, 1, 1, (unsigned)-1, ZPixmap);

    XDestroyImage (tmp_image);
}
